"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectIndexer = void 0;
const path = __importStar(require("path"));
const stream_1 = require("stream");
const pretty_ms_1 = __importDefault(require("pretty-ms"));
const progress_1 = __importDefault(require("progress"));
const ts = __importStar(require("typescript"));
const FileIndexer_1 = require("./FileIndexer");
const Input_1 = require("./Input");
const lsif = __importStar(require("./lsif"));
const Packages_1 = require("./Packages");
class ProjectIndexer {
    constructor(config, options) {
        this.config = config;
        this.symbolCache = new Map();
        this.options = options;
        this.program = ts.createProgram(config.fileNames, config.options);
        this.checker = this.program.getTypeChecker();
        this.packages = new Packages_1.Packages(options.projectRoot);
    }
    index() {
        const startTimestamp = Date.now();
        const sourceFiles = this.program.getSourceFiles();
        const filesToIndex = [];
        // Visit every sourceFile in the program
        for (const sourceFile of sourceFiles) {
            const includes = this.config.fileNames.includes(sourceFile.fileName);
            if (!includes) {
                continue;
            }
            filesToIndex.push(sourceFile);
        }
        if (filesToIndex.length === 0) {
            throw new Error(`no indexable files in project '${this.options.projectDisplayName}'`);
        }
        const jobs = new progress_1.default(`  ${this.options.projectDisplayName} [:bar] :current/:total :title`, {
            total: filesToIndex.length,
            renderThrottle: 100,
            incomplete: '_',
            complete: '#',
            width: 20,
            clear: true,
            stream: this.options.progressBar
                ? process.stderr
                : writableNoopStream(),
        });
        let lastWrite = startTimestamp;
        for (const [index, sourceFile] of filesToIndex.entries()) {
            const title = path.relative(this.options.cwd, sourceFile.fileName);
            jobs.tick({ title });
            if (!this.options.progressBar) {
                const now = Date.now();
                const elapsed = now - lastWrite;
                if (elapsed > 1000 && index > 2) {
                    lastWrite = now;
                    process.stdout.write('.');
                }
            }
            const document = new lsif.lib.codeintel.lsiftyped.Document({
                relative_path: path.relative(this.options.cwd, sourceFile.fileName),
                occurrences: [],
            });
            const input = new Input_1.Input(sourceFile.fileName, sourceFile.getText());
            const visitor = new FileIndexer_1.FileIndexer(this.checker, input, document, this.symbolCache, this.packages, sourceFile);
            try {
                visitor.index();
            }
            catch (error) {
                console.error(`unexpected error indexing project root '${this.options.cwd}'`, error);
            }
            if (visitor.document.occurrences.length > 0) {
                this.options.writeIndex(new lsif.lib.codeintel.lsiftyped.Index({
                    documents: [visitor.document],
                }));
            }
        }
        jobs.terminate();
        const elapsed = Date.now() - startTimestamp;
        if (!this.options.progressBar && lastWrite > startTimestamp) {
            process.stdout.write('\n');
        }
        console.log(`+ ${this.options.projectDisplayName} (${(0, pretty_ms_1.default)(elapsed)})`);
    }
}
exports.ProjectIndexer = ProjectIndexer;
function writableNoopStream() {
    return new stream_1.Writable({
        write(_unused1, _unused2, callback) {
            setImmediate(callback);
        },
    });
}
